@@ -17,6 +17,9 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
+#if !defined (WOCKY_COMPILATION)
+# error "This is an internal header."
+#endif
 
 #ifndef __WOCKY__NODE_PRIVATE_H__
 #define __WOCKY__NODE_PRIVATE_H__
