@@ -25,11 +25,7 @@
 
 #include <telepathy-glib/telepathy-glib.h>
 
-#include <wocky/wocky-xmpp-error.h>
-#include <wocky/wocky-utils.h>
-#include <wocky/wocky-xmpp-reader.h>
-#include <wocky/wocky-xmpp-writer.h>
-#include <wocky/wocky-namespaces.h>
+#include <wocky/wocky.h>
 
 #include "extensions/extensions.h"
 
@@ -77,20 +73,17 @@ gabble_console_plugin_class_init (GabbleConsolePluginClass *klass)
 }
 
 static void
-gabble_console_plugin_create_sidecar (
+gabble_console_plugin_create_sidecar_async (
     GabblePlugin *plugin,
     const gchar *sidecar_interface,
-    GabbleConnection *connection,
+    GabblePluginConnection *connection,
     WockySession *session,
     GAsyncReadyCallback callback,
     gpointer user_data)
 {
   GSimpleAsyncResult *result = g_simple_async_result_new (G_OBJECT (plugin),
       callback, user_data,
-      /* sic: all plugins share gabble_plugin_create_sidecar_finish() so we
-       * need to use the same source tag.
-       */
-      gabble_plugin_create_sidecar);
+      gabble_console_plugin_create_sidecar_async);
   GabbleSidecar *sidecar = NULL;
 
   if (!tp_strdiff (sidecar_interface, GABBLE_IFACE_GABBLE_PLUGIN_CONSOLE))
@@ -114,6 +107,27 @@ gabble_console_plugin_create_sidecar (
   g_object_unref (result);
 }
 
+static GabbleSidecar *
+gabble_console_plugin_create_sidecar_finish (
+    GabblePlugin *plugin,
+    GAsyncResult *result,
+    GError **error)
+{
+  GabbleSidecar *sidecar;
+
+  if (g_simple_async_result_propagate_error (G_SIMPLE_ASYNC_RESULT (result),
+        error))
+    return NULL;
+
+  g_return_val_if_fail (g_simple_async_result_is_valid (result,
+        G_OBJECT (plugin), gabble_console_plugin_create_sidecar_async), NULL);
+
+  sidecar = GABBLE_SIDECAR (g_simple_async_result_get_op_res_gpointer (
+        G_SIMPLE_ASYNC_RESULT (result)));
+  
+  return g_object_ref (sidecar);
+}
+
 static void
 plugin_iface_init (
     gpointer g_iface,
@@ -124,7 +138,8 @@ plugin_iface_init (
   iface->name = "XMPP console";
   iface->version = PACKAGE_VERSION;
   iface->sidecar_interfaces = sidecar_interfaces;
-  iface->create_sidecar = gabble_console_plugin_create_sidecar;
+  iface->create_sidecar_async = gabble_console_plugin_create_sidecar_async;
+  iface->create_sidecar_finish = gabble_console_plugin_create_sidecar_finish;
 }
 
 GabblePlugin *
@@ -296,7 +311,7 @@ gabble_console_sidecar_class_init (GabbleConsoleSidecarClass *klass)
   g_object_class_install_property (object_class, PROP_CONNECTION,
       g_param_spec_object ("connection", "Connection",
           "Gabble connection",
-          GABBLE_TYPE_CONNECTION,
+          GABBLE_TYPE_PLUGIN_CONNECTION,
           G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (object_class, PROP_SESSION,
