@@ -180,7 +180,8 @@ gabble_call_member_content_add_to_session (GabbleCallMemberContent *self)
     content_ns, transport_ns);
 
   content = gabble_jingle_session_add_content (session,
-      priv->media_type, priv->name, content_ns, transport_ns);
+      priv->media_type, JINGLE_CONTENT_SENDERS_BOTH,
+      priv->name, content_ns, transport_ns);
 
   gabble_call_member_content_set_jingle_content (self, content);
 }
@@ -476,7 +477,8 @@ gabble_call_member_content_remove (GabbleCallMemberContent *self)
   g_object_ref (self);
   /* Remove ourselves from the sesison */
   if (priv->jingle_content != NULL)
-      gabble_jingle_content_remove (priv->jingle_content, TRUE);
+      gabble_jingle_session_remove_content (priv->jingle_content->session,
+          priv->jingle_content);
 
   g_signal_emit (self, signals[REMOVED], 0);
   g_object_unref (self);
