@@ -58,7 +58,7 @@ struct _GabbleCallMemberPrivate
   TpHandle target;
 
   GabbleBaseCallChannel *call;
-  TpyCallMemberFlags flags;
+  TpCallMemberFlags flags;
   GabbleJingleSession *session;
 
   GList *contents;
@@ -246,13 +246,13 @@ remote_state_changed_cb (GabbleJingleSession *session, gpointer user_data)
 {
   GabbleCallMember *self = GABBLE_CALL_MEMBER (user_data);
   GabbleCallMemberPrivate *priv = self->priv;
-  TpyCallMemberFlags newflags = 0;
+  TpCallMemberFlags newflags = 0;
 
   if (gabble_jingle_session_get_remote_ringing (session))
-    newflags |= TPY_CALL_MEMBER_FLAG_RINGING;
+    newflags |= TP_CALL_MEMBER_FLAG_RINGING;
 
   if (gabble_jingle_session_get_remote_hold (session))
-    newflags |= TPY_CALL_MEMBER_FLAG_HELD;
+    newflags |= TP_CALL_MEMBER_FLAG_HELD;
 
   if (priv->flags == newflags)
     return;
@@ -385,7 +385,7 @@ gabble_call_member_get_session (GabbleCallMember *self)
   return self->priv->session;
 }
 
-TpyCallMemberFlags
+TpCallMemberFlags
 gabble_call_member_get_flags (GabbleCallMember *self)
 {
   return self->priv->flags;
@@ -439,6 +439,7 @@ GabbleCallMemberContent *
 gabble_call_member_create_content (GabbleCallMember *self,
   const gchar *name,
   JingleMediaType mtype,
+  JingleContentSenders senders,
   GError **error)
 {
   GabbleCallMemberPrivate *priv = self->priv;
@@ -473,7 +474,7 @@ gabble_call_member_create_content (GabbleCallMember *self,
     content_ns, priv->transport_ns);
 
   c = gabble_jingle_session_add_content (priv->session,
-      mtype, name, content_ns, priv->transport_ns);
+      mtype, senders, name, content_ns, priv->transport_ns);
 
   g_assert (c != NULL);
 
@@ -504,16 +505,18 @@ gabble_call_member_open_session (GabbleCallMember *self,
 {
   GabbleCallMemberPrivate *priv = self->priv;
   GabbleConnection *conn = gabble_call_member_get_connection (self);
+  GabbleJingleFactory *jf;
   GabbleJingleSession *session;
   gchar *jid;
 
   jid = gabble_peer_to_jid (conn, priv->target, NULL);
 
-  session = gabble_jingle_factory_create_session (conn->jingle_factory,
-      priv->target, jid, FALSE);
-  DEBUG ("Created a jingle session: %p", session);
+  jf = gabble_jingle_mint_get_factory (conn->jingle_mint);
+  g_return_val_if_fail (jf != NULL, FALSE);
 
-  g_object_set (session, "dialect", JINGLE_DIALECT_V032, NULL);
+  session = gabble_jingle_factory_create_session (jf, jid, JINGLE_DIALECT_V032,
+      FALSE);
+  DEBUG ("Created a jingle session: %p", session);
 
   priv->transport_ns = g_strdup (NS_JINGLE_TRANSPORT_ICEUDP);
 
@@ -537,6 +540,7 @@ gabble_call_member_start_session (GabbleCallMember *self,
   JingleDialect dialect;
   gchar *jid;
   const gchar *transport;
+  GabbleJingleFactory *jf;
   GabbleJingleSession *session;
 
   /* FIXME might need to wait on capabilities, also don't need transport
@@ -552,22 +556,24 @@ gabble_call_member_start_session (GabbleCallMember *self,
 
   jid = gabble_peer_to_jid (gabble_call_member_get_connection (self), target, resource);
 
-  session = gabble_jingle_factory_create_session (
-        gabble_call_member_get_connection (self)->jingle_factory, target, jid, FALSE);
+  jf = gabble_jingle_mint_get_factory (
+        gabble_call_member_get_connection (self)->jingle_mint);
+  g_return_val_if_fail (jf != NULL, FALSE);
+
+  session = gabble_jingle_factory_create_session (jf, jid, dialect, FALSE);
   g_free (jid);
 
   gabble_call_member_set_session (self, session);
-  g_object_set (session, "dialect", dialect, NULL);
 
   priv->transport_ns = g_strdup (transport);
 
   if (audio_name != NULL)
     gabble_call_member_create_content (self, audio_name,
-      JINGLE_MEDIA_TYPE_AUDIO, NULL);
+      JINGLE_MEDIA_TYPE_AUDIO, JINGLE_CONTENT_SENDERS_BOTH, NULL);
 
   if (video_name != NULL)
     gabble_call_member_create_content (self, video_name,
-      JINGLE_MEDIA_TYPE_VIDEO, NULL);
+      JINGLE_MEDIA_TYPE_VIDEO, JINGLE_CONTENT_SENDERS_BOTH, NULL);
 
   return TRUE;
 }
