@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "config.h"
+
 #define _BSD_SOURCE
 #define _XOPEN_SOURCE /* glibc2 needs this */
 #include <time.h>
@@ -28,21 +30,23 @@
 #include <string.h>
 #include <glib/gstdio.h>
 
+#ifdef ENABLE_JINGLE_FILE_TRANSFER
 #include "jingle-session.h"
 #include "jingle-share.h"
+#endif
 #include "gabble/caps-channel-manager.h"
 #include "connection.h"
 #include "ft-manager.h"
 #include "ft-channel.h"
-#include "error.h"
 #include "gabble-signals-marshal.h"
 #include "namespaces.h"
 #include "presence-cache.h"
 #include "util.h"
 
-#include <wocky/wocky-data-form.h>
+#include <wocky/wocky.h>
 
 #include <telepathy-glib/base-connection.h>
+#include <telepathy-glib/base-channel.h>
 #include <telepathy-glib/channel-factory-iface.h>
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/gtypes.h>
@@ -191,7 +195,7 @@ ft_manager_close_all (GabbleFtManager *self)
 
   while ((l = self->priv->channels) != NULL)
     {
-      gabble_file_transfer_channel_do_close (l->data);
+      tp_base_channel_close (l->data);
       /* Channels should have closed and disappeared from the list */
       g_assert (l != self->priv->channels);
     }
@@ -287,6 +291,7 @@ file_channel_closed_cb (GabbleFileTransferChannel *chan,
     }
 }
 
+#ifdef ENABLE_JINGLE_FILE_TRANSFER
 static void
 gabble_ft_manager_channels_created (GabbleFtManager *self, GList *channels)
 {
@@ -298,6 +303,8 @@ gabble_ft_manager_channels_created (GabbleFtManager *self, GList *channels)
     {
       GabbleFileTransferChannel *chan = i->data;
 
+      tp_base_channel_register (TP_BASE_CHANNEL (chan));
+
       gabble_signal_connect_weak (chan, "closed",
           G_CALLBACK (file_channel_closed_cb), G_OBJECT (self));
 
@@ -311,6 +318,7 @@ gabble_ft_manager_channels_created (GabbleFtManager *self, GList *channels)
 
   g_hash_table_unref (new_channels);
 }
+#endif
 
 static void
 gabble_ft_manager_channel_created (GabbleFtManager *self,
@@ -319,6 +327,8 @@ gabble_ft_manager_channel_created (GabbleFtManager *self,
 {
   GSList *requests = NULL;
 
+  tp_base_channel_register (TP_BASE_CHANNEL (chan));
+
   gabble_signal_connect_weak (chan, "closed",
       G_CALLBACK (file_channel_closed_cb), G_OBJECT (self));
 
@@ -333,9 +343,9 @@ gabble_ft_manager_channel_created (GabbleFtManager *self,
   g_slist_free (requests);
 }
 
-
+#ifdef ENABLE_JINGLE_FILE_TRANSFER
 static void
-new_jingle_session_cb (GabbleJingleFactory *jf,
+new_jingle_session_cb (GabbleJingleMint *jm,
     GabbleJingleSession *sess,
     gpointer data)
 {
@@ -360,7 +370,9 @@ new_jingle_session_cb (GabbleJingleFactory *jf,
       if (content == NULL)
           return;
 
-      gtalk_fc = gtalk_file_collection_new_from_session (jf, sess);
+      gtalk_fc = gtalk_file_collection_new_from_session (
+          gabble_jingle_mint_get_factory (jm),
+          sess);
 
       if (gtalk_fc)
         {
@@ -377,11 +389,16 @@ new_jingle_session_cb (GabbleJingleFactory *jf,
               GabbleJingleShareManifestEntry *entry = i->data;
               GabbleFileTransferChannel *channel = NULL;
               gchar *filename = NULL;
+              TpHandleRepoIface *contacts = tp_base_connection_get_handles (
+                  TP_BASE_CONNECTION (self->priv->connection),
+                  TP_HANDLE_TYPE_CONTACT);
+              TpHandle peer = tp_handle_ensure (contacts,
+                  gabble_jingle_session_get_peer_jid (sess), NULL, NULL);
 
               filename = g_strdup_printf ("%s%s",
                   entry->name, entry->folder? ".tar":"");
               channel = gabble_file_transfer_channel_new (self->priv->connection,
-                  sess->peer, sess->peer, TP_FILE_TRANSFER_STATE_PENDING,
+                  peer, peer, TP_FILE_TRANSFER_STATE_PENDING,
                   NULL, filename, entry->size, TP_FILE_HASH_TYPE_NONE, NULL,
                   NULL, 0, 0, FALSE, NULL, gtalk_fc, token, NULL, NULL, NULL);
               g_free (filename);
@@ -402,7 +419,7 @@ new_jingle_session_cb (GabbleJingleFactory *jf,
         }
     }
 }
-
+#endif
 
 static void
 connection_status_changed_cb (GabbleConnection *conn,
@@ -413,11 +430,13 @@ connection_status_changed_cb (GabbleConnection *conn,
 
   switch (status)
     {
+#ifdef ENABLE_JINGLE_FILE_TRANSFER
       case TP_CONNECTION_STATUS_CONNECTING:
-        g_signal_connect (self->priv->connection->jingle_factory,
-            "new-session",
+        g_signal_connect (self->priv->connection->jingle_mint,
+            "incoming-session",
             G_CALLBACK (new_jingle_session_cb), self);
         break;
+#endif
 
       case TP_CONNECTION_STATUS_DISCONNECTED:
         ft_manager_close_all (self);
@@ -666,9 +685,9 @@ gabble_ft_manager_type_foreach_channel_class (GType type,
   g_hash_table_unref (table);
 }
 
-static LmMessageNode *
+static WockyNode *
 hyvaa_vappua (
-    LmMessageNode *si_node,
+    WockyNode *si_node,
     const gchar **filename,
     const gchar **size_str,
     GError **error)
@@ -676,17 +695,18 @@ hyvaa_vappua (
 #define die_if_null(var, msg) \
   if ((var) == NULL) \
     { \
-      g_set_error (error, GABBLE_XMPP_ERROR, XMPP_ERROR_BAD_REQUEST, msg); \
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST, \
+          msg); \
       return NULL; \
     }
 
-  LmMessageNode *file_node = lm_message_node_get_child_with_namespace (si_node,
+  WockyNode *file_node = wocky_node_get_child_ns (si_node,
       "file", NS_FILE_TRANSFER);
 
   die_if_null (file_node, "Invalid file transfer SI request: no <file>")
-  die_if_null (*filename = lm_message_node_get_attribute (file_node, "name"),
+  die_if_null (*filename = wocky_node_get_attribute (file_node, "name"),
       "Invalid file transfer SI request: missing file name")
-  die_if_null (*size_str = lm_message_node_get_attribute (file_node, "size"),
+  die_if_null (*size_str = wocky_node_get_attribute (file_node, "size"),
       "Invalid file transfer SI request: missing file size")
 
   return file_node;
@@ -747,7 +767,7 @@ extract_service_name (WockyNode *file)
 
   if (field == NULL)
     {
-      DEBUG ("ServiceName propery not present in data form; odd...");
+      DEBUG ("ServiceName property not present in data form; odd...");
       goto out;
     }
 
@@ -802,9 +822,9 @@ void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
                                           GabbleBytestreamIface *bytestream,
                                           TpHandle handle,
                                           const gchar *stream_id,
-                                          LmMessage *msg)
+                                          WockyStanza *msg)
 {
-  LmMessageNode *si_node, *file_node, *desc_node;
+  WockyNode *si_node, *file_node, *desc_node;
   const gchar *filename, *size_str, *content_type, *content_hash, *description;
   const gchar *date_str;
   gchar *service_name;
@@ -816,7 +836,7 @@ void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
   gboolean resume_supported;
   GError *error = NULL;
 
-  si_node = lm_message_node_get_child_with_namespace (
+  si_node = wocky_node_get_child_ns (
       wocky_stanza_get_top_node (msg), "si", NS_SI);
   g_assert (si_node != NULL);
 
@@ -832,24 +852,24 @@ void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
 
   size = g_ascii_strtoull (size_str, NULL, 0);
 
-  content_type = lm_message_node_get_attribute (file_node, "mime-type");
+  content_type = wocky_node_get_attribute (file_node, "mime-type");
   if (content_type == NULL)
     content_type = "application/octet-stream";
 
   /* The hash is always an MD5-sum, if present. */
-  content_hash = lm_message_node_get_attribute (file_node, "hash");
+  content_hash = wocky_node_get_attribute (file_node, "hash");
   if (content_hash != NULL)
     content_hash_type = TP_FILE_HASH_TYPE_MD5;
   else
     content_hash_type = TP_FILE_HASH_TYPE_NONE;
 
-  desc_node = lm_message_node_get_child (file_node, "desc");
+  desc_node = wocky_node_get_child (file_node, "desc");
   if (desc_node != NULL)
-    description = lm_message_node_get_value (desc_node);
+    description = desc_node->content;
   else
     description = NULL;
 
-  date_str = lm_message_node_get_attribute (file_node, "date");
+  date_str = wocky_node_get_attribute (file_node, "date");
   if (date_str != NULL)
     {
       GTimeVal val;
@@ -859,7 +879,7 @@ void gabble_ft_manager_handle_si_request (GabbleFtManager *self,
         date = val.tv_sec;
     }
 
-  resume_supported = (lm_message_node_get_child (file_node, "range") != NULL);
+  resume_supported = (wocky_node_get_child (file_node, "range") != NULL);
 
   /* metadata */
   service_name = extract_service_name (file_node);
