@@ -851,63 +851,25 @@ typedef struct
   ShareChannel *share_channel;
 } GoogleRelaySessionData;
 
+static const NiceRelayType relay_type_map[] = {
+    /* GABBLE_JINGLE_RELAY_TYPE_UDP */ NICE_RELAY_TYPE_TURN_UDP,
+    /* GABBLE_JINGLE_RELAY_TYPE_TCP */ NICE_RELAY_TYPE_TURN_TCP,
+    /* GABBLE_JINGLE_RELAY_TYPE_TLS */ NICE_RELAY_TYPE_TURN_TLS,
+};
+
 static void
 set_relay_info (gpointer item, gpointer user_data)
 {
   GoogleRelaySessionData *data = user_data;
-  GHashTable *relay = item;
-  const gchar *server_ip = NULL;
-  const gchar *username = NULL;
-  const gchar *password = NULL;
-  const gchar *type_str = NULL;
-  guint server_port;
+  GabbleJingleRelay *relay = item;
   NiceRelayType type;
-  GValue *value;
 
-  value = g_hash_table_lookup (relay, "ip");
-  if (value != NULL)
-    server_ip = g_value_get_string (value);
-  else
-    return;
-
-  value = g_hash_table_lookup (relay, "port");
-  if (value != NULL)
-    server_port = g_value_get_uint (value);
-  else
-    return;
-
-  value = g_hash_table_lookup (relay, "username");
-  if (value != NULL)
-    username = g_value_get_string (value);
-  else
-    return;
-
-  value = g_hash_table_lookup (relay, "password");
-  if (value != NULL)
-    password = g_value_get_string (value);
-  else
-    return;
-
-  value = g_hash_table_lookup (relay, "type");
-  if (value != NULL)
-    type_str = g_value_get_string (value);
-  else
-    return;
-
-  if (!strcmp (type_str, "udp"))
-    type = NICE_RELAY_TYPE_TURN_UDP;
-  else if (!strcmp (type_str, "tcp"))
-    type = NICE_RELAY_TYPE_TURN_TCP;
-  else if (!strcmp (type_str, "tls"))
-    type = NICE_RELAY_TYPE_TURN_TLS;
-  else
-    return;
+  g_return_if_fail (relay->type < GABBLE_N_JINGLE_RELAY_TYPES);
+  type = relay_type_map[relay->type];
 
   nice_agent_set_relay_info (data->share_channel->agent,
       data->share_channel->stream_id, data->share_channel->component_id,
-      server_ip, server_port,
-      username, password, type);
-
+      relay->ip, relay->port, relay->username, relay->password, type);
 }
 
 static void
@@ -983,8 +945,9 @@ content_new_share_channel_cb (GabbleJingleContent *content, const gchar *name,
   nice_agent_attach_recv (agent, stream_id, share_channel->component_id,
       g_main_context_default (), nice_data_received_cb, self);
 
-  if (gabble_jingle_factory_get_stun_server (
-          self->priv->jingle_factory, &stun_server, &stun_port))
+  if (gabble_jingle_info_get_stun_server (
+          gabble_jingle_factory_get_jingle_info (self->priv->jingle_factory),
+          &stun_server, &stun_port))
     {
       g_object_set (agent,
           "stun-server", stun_server,
@@ -998,8 +961,8 @@ content_new_share_channel_cb (GabbleJingleContent *content, const gchar *name,
   relay_data->share_channel = share_channel;
   g_object_add_weak_pointer (G_OBJECT (relay_data->u.self),
       &relay_data->u.ptr);
-  gabble_jingle_factory_create_google_relay_session (
-      self->priv->jingle_factory, 1,
+  gabble_jingle_info_create_google_relay_session (
+      gabble_jingle_factory_get_jingle_info (self->priv->jingle_factory), 1,
       google_relay_session_cb, relay_data);
 }
 
@@ -1449,7 +1412,7 @@ gtalk_file_collection_new (GabbleFileTransferChannel *channel,
   self->priv->requested = TRUE;
 
   session = gabble_jingle_factory_create_session (jingle_factory,
-      handle, jid, FALSE);
+      jid, JINGLE_DIALECT_GTALK4, FALSE);
 
   if (session == NULL)
     {
@@ -1457,13 +1420,9 @@ gtalk_file_collection_new (GabbleFileTransferChannel *channel,
       return NULL;
     }
 
-  g_object_set (session,
-      "dialect", JINGLE_DIALECT_GTALK4,
-      NULL);
-
   content = gabble_jingle_session_add_content (session,
-      JINGLE_MEDIA_TYPE_NONE, "share", NS_GOOGLE_SESSION_SHARE,
-      NS_GOOGLE_TRANSPORT_P2P);
+      JINGLE_MEDIA_TYPE_NONE, JINGLE_CONTENT_SENDERS_BOTH, "share",
+      NS_GOOGLE_SESSION_SHARE, NS_GOOGLE_TRANSPORT_P2P);
 
   if (content == NULL)
     {
