@@ -24,7 +24,6 @@
 #include <string.h>
 
 #include <dbus/dbus-glib.h>
-#include <loudmouth/loudmouth.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/errors.h>
@@ -249,7 +248,7 @@ im_channel_send_gone (GabbleIMChannel *self)
       if (chat_states_supported (self, FALSE))
         gabble_message_util_send_chat_state (G_OBJECT (self),
             GABBLE_CONNECTION (tp_base_channel_get_connection (base)),
-            LM_MESSAGE_SUB_TYPE_CHAT, TP_CHANNEL_CHAT_STATE_GONE,
+            WOCKY_STANZA_SUB_TYPE_CHAT, TP_CHANNEL_CHAT_STATE_GONE,
             priv->peer_jid, NULL);
 
       priv->send_gone = FALSE;
@@ -650,7 +649,7 @@ gabble_im_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
 
       if (gabble_message_util_send_chat_state (G_OBJECT (self),
               GABBLE_CONNECTION (base_conn),
-              LM_MESSAGE_SUB_TYPE_CHAT, state, priv->peer_jid, &error))
+              WOCKY_STANZA_SUB_TYPE_CHAT, state, priv->peer_jid, &error))
         {
           priv->send_gone = TRUE;
 
