@@ -21,10 +21,10 @@
 #define __JINGLE_CONTENT_H__
 
 #include <glib-object.h>
-#include <loudmouth/loudmouth.h>
-#include "types.h"
+
 #include "jingle-factory.h"
 #include "jingle-transport-iface.h"
+#include "jingle-types.h"
 
 G_BEGIN_DECLS
 
@@ -52,7 +52,7 @@ struct _JingleCandidate {
   int component;
   int generation;
 
-  gdouble preference;
+  int preference;
   gchar *username;
   gchar *password;
   int network;
@@ -82,9 +82,9 @@ GType gabble_jingle_content_get_type (void);
 struct _GabbleJingleContentClass {
     GObjectClass parent_class;
 
-    void  (*parse_description) (GabbleJingleContent *, LmMessageNode *,
+    void  (*parse_description) (GabbleJingleContent *, WockyNode *,
         GError **);
-    void  (*produce_description) (GabbleJingleContent *, LmMessageNode *);
+    void  (*produce_description) (GabbleJingleContent *, WockyNode *);
     void  (*transport_created) (GabbleJingleContent *,
         GabbleJingleTransportIface *);
     JingleContentSenders (*get_default_senders) (GabbleJingleContent *);
@@ -96,28 +96,27 @@ struct _GabbleJingleContent {
     GObject parent;
     GabbleJingleContentPrivate *priv;
 
-    GabbleConnection *conn;
     GabbleJingleSession *session;
 };
 
 void gabble_jingle_content_parse_add (GabbleJingleContent *c,
-    LmMessageNode *content_node, gboolean google_mode, GError **error);
+    WockyNode *content_node, gboolean google_mode, GError **error);
 void gabble_jingle_content_update_senders (GabbleJingleContent *c,
-    LmMessageNode *content_node, GError **error);
+    WockyNode *content_node, GError **error);
 void gabble_jingle_content_produce_node (GabbleJingleContent *c,
-    LmMessageNode *parent,
+    WockyNode *parent,
     gboolean include_description,
     gboolean include_transport,
-    LmMessageNode **trans_node_out);
+    WockyNode **trans_node_out);
 void gabble_jingle_content_parse_accept (GabbleJingleContent *c,
-  LmMessageNode *content_node, gboolean google_mode, GError **error);
+  WockyNode *content_node, gboolean google_mode, GError **error);
 
 void gabble_jingle_content_parse_info (GabbleJingleContent *c,
-    LmMessageNode *content_node, GError **error);
+    WockyNode *content_node, GError **error);
 void gabble_jingle_content_parse_transport_info (GabbleJingleContent *self,
-  LmMessageNode *trans_node, GError **error);
+  WockyNode *trans_node, GError **error);
 void gabble_jingle_content_parse_description_info (GabbleJingleContent *self,
-  LmMessageNode *trans_node, GError **error);
+  WockyNode *trans_node, GError **error);
 guint gabble_jingle_content_create_share_channel (GabbleJingleContent *self,
     const gchar *name);
 void gabble_jingle_content_add_candidates (GabbleJingleContent *self, GList *li);
@@ -126,14 +125,19 @@ gboolean gabble_jingle_content_is_ready (GabbleJingleContent *self);
 void gabble_jingle_content_set_transport_state (GabbleJingleContent *content,
     JingleTransportState state);
 void gabble_jingle_content_remove (GabbleJingleContent *c, gboolean signal_peer);
+void gabble_jingle_content_reject (GabbleJingleContent *c,
+    JingleReason reason);
+
 GList *gabble_jingle_content_get_remote_candidates (GabbleJingleContent *c);
 GList *gabble_jingle_content_get_local_candidates (GabbleJingleContent *c);
+gboolean gabble_jingle_content_get_credentials (GabbleJingleContent *c,
+  gchar **ufrag, gchar **pwd);
 gboolean gabble_jingle_content_change_direction (GabbleJingleContent *c,
     JingleContentSenders senders);
 void gabble_jingle_content_retransmit_candidates (GabbleJingleContent *self,
     gboolean all);
 void gabble_jingle_content_inject_candidates (GabbleJingleContent *self,
-    LmMessageNode *transport_node);
+    WockyNode *transport_node);
 gboolean gabble_jingle_content_is_created_by_us (GabbleJingleContent *c);
 gboolean gabble_jingle_content_creator_is_initiator (GabbleJingleContent *c);
 
@@ -149,7 +153,9 @@ gboolean gabble_jingle_content_sending (GabbleJingleContent *self);
 gboolean gabble_jingle_content_receiving (GabbleJingleContent *self);
 
 void gabble_jingle_content_set_sending (GabbleJingleContent *self,
-  gboolean send);
+    gboolean send);
+void gabble_jingle_content_request_receiving (GabbleJingleContent *self,
+    gboolean receive);
 
 void gabble_jingle_content_send_complete (GabbleJingleContent *self);
 
