@@ -21,9 +21,8 @@
 #define __JINGLE_TRANSPORT_GOOGLE_H__
 
 #include <glib-object.h>
-#include <loudmouth/loudmouth.h>
 
-#include "types.h"
+#include "jingle-types.h"
 
 G_BEGIN_DECLS
 
