@@ -25,13 +25,10 @@
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-#include <loudmouth/loudmouth.h>
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/interfaces.h>
 
-#include <telepathy-yell/interfaces.h>
-
 #define DEBUG_FLAG GABBLE_DEBUG_MEDIA
 
 #include "gabble/caps-channel-manager.h"
@@ -378,7 +375,7 @@ new_call_channel (GabbleMediaFactory *self,
   TpHandle initiator;
 
   if (sess != NULL)
-    initiator = sess->peer;
+    initiator = peer;
   else
     initiator = conn->self_handle;
 
@@ -399,6 +396,7 @@ new_call_channel (GabbleMediaFactory *self,
         initial_video_name != NULL ? initial_video_name : "video",
     "requested", request_token != NULL,
     "initiator-handle", initiator,
+    "mutable-contents", TRUE,
     NULL);
 
   g_free (object_path);
@@ -443,12 +441,14 @@ gabble_media_factory_close_all (GabbleMediaFactory *fac)
 }
 
 static void
-new_jingle_session_cb (GabbleJingleFactory *jf,
+new_jingle_session_cb (GabbleJingleMint *jm,
     GabbleJingleSession *sess,
     gpointer data)
 {
   GabbleMediaFactory *self = GABBLE_MEDIA_FACTORY (data);
   GabbleMediaFactoryPrivate *priv = self->priv;
+  TpHandleRepoIface *contacts;
+  TpHandle peer;
 
   if (gabble_jingle_session_get_content_type (sess) !=
       GABBLE_TYPE_JINGLE_MEDIA_RTP)
@@ -457,17 +457,25 @@ new_jingle_session_cb (GabbleJingleFactory *jf,
   if (gabble_muc_factory_handle_jingle_session (priv->conn->muc_factory, sess))
     {
       /* Muji channel the muc factory is taking care of it */
+      return;
     }
-  else if (self->priv->use_call_channels)
+
+  contacts = tp_base_connection_get_handles (TP_BASE_CONNECTION (priv->conn),
+      TP_HANDLE_TYPE_CONTACT);
+  peer = tp_handle_ensure (contacts, gabble_jingle_session_get_peer_jid (sess),
+      NULL, NULL);
+
+  if (self->priv->use_call_channels)
     {
-      new_call_channel (self, sess, sess->peer,
+      new_call_channel (self, sess, peer,
         FALSE, NULL,
         FALSE, NULL,
         NULL);
     }
   else
     {
-      GabbleMediaChannel *chan = new_media_channel (self, sess, sess->peer,
+      GabbleMediaChannel *chan = new_media_channel (self, sess,
+          peer,
           FALSE, FALSE, FALSE);
       GList *cs;
 
@@ -507,7 +515,7 @@ connection_status_changed_cb (GabbleConnection *conn,
   switch (status)
     {
     case TP_CONNECTION_STATUS_CONNECTING:
-      g_signal_connect (priv->conn->jingle_factory, "new-session",
+      g_signal_connect (priv->conn->jingle_mint, "incoming-session",
           G_CALLBACK (new_jingle_session_cb), self);
       break;
 
@@ -551,18 +559,18 @@ gabble_media_factory_foreach_channel (TpChannelManager *manager,
 }
 
 static const gchar * const media_channel_fixed_properties[] = {
-    TP_IFACE_CHANNEL ".ChannelType",
-    TP_IFACE_CHANNEL ".TargetHandleType",
+    TP_PROP_CHANNEL_CHANNEL_TYPE,
+    TP_PROP_CHANNEL_TARGET_HANDLE_TYPE,
     NULL
 };
 
 /* If you change this at all, you'll probably also need to change both_allowed
  * and video_allowed */
 static const gchar * const named_channel_allowed_properties[] = {
-    TP_IFACE_CHANNEL ".TargetHandle",
-    TP_IFACE_CHANNEL ".TargetID",
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio",
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo",
+    TP_PROP_CHANNEL_TARGET_HANDLE,
+    TP_PROP_CHANNEL_TARGET_ID,
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO,
     NULL
 };
 
@@ -570,61 +578,61 @@ static const gchar * const * both_allowed =
     named_channel_allowed_properties + 2;
 
 static const gchar * const audio_allowed[] = {
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio",
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".ImmutableStreams",
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_IMMUTABLE_STREAMS,
     NULL
 };
 
 static const gchar * const video_allowed[] = {
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo",
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".ImmutableStreams",
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO,
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_IMMUTABLE_STREAMS,
     NULL
 };
 
 static const gchar * const both_allowed_immutable[] = {
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio",
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo",
-    TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".ImmutableStreams",
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO,
+    TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_IMMUTABLE_STREAMS,
     NULL
 };
 
 static const gchar * const call_channel_allowed_properties[] = {
-    TP_IFACE_CHANNEL ".TargetHandle",
-    TP_IFACE_CHANNEL ".TargetID",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudio",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudioName",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideo",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideoName",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".MutableContents",
+    TP_PROP_CHANNEL_TARGET_HANDLE,
+    TP_PROP_CHANNEL_TARGET_ID,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO_NAME,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO_NAME,
+    TP_PROP_CHANNEL_TYPE_CALL_MUTABLE_CONTENTS,
     NULL
 };
 
 static const gchar * const call_audio_allowed[] = {
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudio",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudioName",
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO_NAME,
     NULL
 };
 
 static const gchar * const call_video_allowed[] = {
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideo",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideoName",
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO_NAME,
     NULL
 };
 
 static const gchar * const call_both_allowed[] = {
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudio",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudioName",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideo",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideoName",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".MutableContents",
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO_NAME,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO_NAME,
+    TP_PROP_CHANNEL_TYPE_CALL_MUTABLE_CONTENTS,
     NULL
 };
 
 static const gchar * const call_both_allowed_immutable[] = {
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudio",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudioName",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideo",
-    TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideoName",
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO_NAME,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO,
+    TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO_NAME,
     NULL
 };
 
@@ -644,12 +652,12 @@ static GHashTable *
 gabble_media_factory_streamed_media_channel_class (void)
 {
   GHashTable *table = tp_asv_new (
-      TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_UINT,
+      TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT,
           TP_HANDLE_TYPE_CONTACT,
       NULL);
 
   tp_asv_set_static_string (table,
-      TP_IFACE_CHANNEL ".ChannelType",
+      TP_PROP_CHANNEL_CHANNEL_TYPE,
       TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA);
 
   return table;
@@ -659,13 +667,12 @@ static GHashTable *
 gabble_media_factory_call_channel_class (void)
 {
   GHashTable *table = tp_asv_new (
-      TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_UINT,
+      TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, G_TYPE_UINT,
           TP_HANDLE_TYPE_CONTACT,
       NULL);
 
-  tp_asv_set_static_string (table,
-      TP_IFACE_CHANNEL ".ChannelType",
-      TPY_IFACE_CHANNEL_TYPE_CALL);
+  tp_asv_set_static_string (table, TP_PROP_CHANNEL_CHANNEL_TYPE,
+      TP_IFACE_CHANNEL_TYPE_CALL);
 
   return table;
 }
@@ -746,20 +753,20 @@ gabble_media_factory_requestotron (TpChannelManager *manager,
     }
 
   if (tp_strdiff (tp_asv_get_string (request_properties,
-          TP_IFACE_CHANNEL ".ChannelType"),
+          TP_PROP_CHANNEL_CHANNEL_TYPE),
         TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA))
     return FALSE;
 
   handle_type = tp_asv_get_uint32 (request_properties,
-      TP_IFACE_CHANNEL ".TargetHandleType", NULL);
+      TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, NULL);
 
   handle = tp_asv_get_uint32 (request_properties,
-      TP_IFACE_CHANNEL ".TargetHandle", NULL);
+      TP_PROP_CHANNEL_TARGET_HANDLE, NULL);
 
   initial_audio = tp_asv_get_boolean (request_properties,
-      TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio", NULL);
+      TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO, NULL);
   initial_video = tp_asv_get_boolean (request_properties,
-      TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo", NULL);
+      TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO, NULL);
 
   switch (handle_type)
     {
@@ -854,7 +861,7 @@ gabble_media_factory_create_call (TpChannelManager *manager,
   DEBUG ("Creating a new call channel");
 
   if (tp_asv_get_uint32 (request_properties,
-      TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_CONTACT)
+      TP_PROP_CHANNEL_TARGET_HANDLE_TYPE, NULL) != TP_HANDLE_TYPE_CONTACT)
     return FALSE;
 
   if (tp_channel_manager_asv_has_unknown_properties (request_properties,
@@ -863,7 +870,7 @@ gabble_media_factory_create_call (TpChannelManager *manager,
         goto error;
 
   target  = tp_asv_get_uint32 (request_properties,
-      TP_IFACE_CHANNEL ".TargetHandle", NULL);
+      TP_PROP_CHANNEL_TARGET_HANDLE, NULL);
 
   if (method == METHOD_ENSURE)
     {
@@ -905,9 +912,9 @@ gabble_media_factory_create_call (TpChannelManager *manager,
     }
 
   initial_audio = tp_asv_get_boolean (request_properties,
-      TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudio", NULL);
+      TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL);
   initial_video = tp_asv_get_boolean (request_properties,
-      TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideo", NULL);
+      TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL);
 
   if (!initial_audio && !initial_video)
     {
@@ -922,9 +929,9 @@ gabble_media_factory_create_call (TpChannelManager *manager,
    */
 
   initial_audio_name = tp_asv_get_string (request_properties,
-    TPY_IFACE_CHANNEL_TYPE_CALL "InitialAudioName");
+      TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO_NAME);
   initial_video_name = tp_asv_get_string (request_properties,
-    TPY_IFACE_CHANNEL_TYPE_CALL "InitialVideoName");
+      TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO_NAME);
 
   new_call_channel (self, NULL, target,
     initial_audio, initial_audio_name,
@@ -956,8 +963,8 @@ gabble_media_factory_create_channel (TpChannelManager *manager,
                                      GHashTable *request_properties)
 {
   if (!tp_strdiff (tp_asv_get_string (request_properties,
-          TP_IFACE_CHANNEL ".ChannelType"),
-        TPY_IFACE_CHANNEL_TYPE_CALL))
+          TP_PROP_CHANNEL_CHANNEL_TYPE),
+        TP_IFACE_CHANNEL_TYPE_CALL))
     return gabble_media_factory_create_call (manager, request_token,
       request_properties, METHOD_CREATE);
   else
@@ -972,8 +979,8 @@ gabble_media_factory_ensure_channel (TpChannelManager *manager,
                                      GHashTable *request_properties)
 {
   if (!tp_strdiff (tp_asv_get_string (request_properties,
-          TP_IFACE_CHANNEL ".ChannelType"),
-        TPY_IFACE_CHANNEL_TYPE_CALL))
+          TP_PROP_CHANNEL_CHANNEL_TYPE),
+        TP_IFACE_CHANNEL_TYPE_CALL))
     return gabble_media_factory_create_call (manager, request_token,
         request_properties, METHOD_ENSURE);
   else
@@ -1221,15 +1228,15 @@ gabble_media_factory_represent_client (GabbleCapsChannelManager *manager,
       q_gtalk_p2p = g_quark_from_static_string (
           TP_IFACE_CHANNEL_INTERFACE_MEDIA_SIGNALLING "/gtalk-p2p");
       qc_gtalk_p2p = g_quark_from_static_string (
-          TPY_IFACE_CHANNEL_TYPE_CALL "/gtalk-p2p");
+          TP_IFACE_CHANNEL_TYPE_CALL "/gtalk-p2p");
       q_ice_udp = g_quark_from_static_string (
           TP_IFACE_CHANNEL_INTERFACE_MEDIA_SIGNALLING "/ice-udp");
       qc_ice_udp = g_quark_from_static_string (
-          TPY_IFACE_CHANNEL_TYPE_CALL "/ice-udp");
+          TP_IFACE_CHANNEL_TYPE_CALL "/ice-udp");
       q_h264 = g_quark_from_static_string (
           TP_IFACE_CHANNEL_INTERFACE_MEDIA_SIGNALLING "/video/h264");
       qc_h264 = g_quark_from_static_string (
-          TPY_IFACE_CHANNEL_TYPE_CALL "/video/h264");
+          TP_IFACE_CHANNEL_TYPE_CALL "/video/h264");
     }
 
   if (cap_tokens != NULL)
@@ -1271,11 +1278,11 @@ gabble_media_factory_represent_client (GabbleCapsChannelManager *manager,
         }
 
       if (tp_strdiff (tp_asv_get_string (filter,
-              TP_IFACE_CHANNEL ".ChannelType"),
+              TP_PROP_CHANNEL_CHANNEL_TYPE),
             TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA)
           && tp_strdiff (tp_asv_get_string (filter,
-              TP_IFACE_CHANNEL ".ChannelType"),
-            TPY_IFACE_CHANNEL_TYPE_CALL))
+              TP_PROP_CHANNEL_CHANNEL_TYPE),
+            TP_IFACE_CHANNEL_TYPE_CALL))
         {
           /* not interesting to this channel manager */
           continue;
@@ -1285,17 +1292,17 @@ gabble_media_factory_represent_client (GabbleCapsChannelManager *manager,
       /* If there is a handler that can support Call channels, use those for
        * incoming channels */
       if (!tp_strdiff (tp_asv_get_string (filter,
-            TP_IFACE_CHANNEL ".ChannelType"),
-            TPY_IFACE_CHANNEL_TYPE_CALL))
+            TP_PROP_CHANNEL_CHANNEL_TYPE),
+            TP_IFACE_CHANNEL_TYPE_CALL))
         {
           GabbleMediaFactory *self = GABBLE_MEDIA_FACTORY (manager);
           self->priv->use_call_channels = TRUE;
         }
 #endif
 
-      if (tp_asv_lookup (filter, TP_IFACE_CHANNEL ".TargetHandleType")
+      if (tp_asv_lookup (filter, TP_PROP_CHANNEL_TARGET_HANDLE_TYPE)
           != NULL &&
-          tp_asv_get_uint32 (filter, TP_IFACE_CHANNEL ".TargetHandleType",
+          tp_asv_get_uint32 (filter, TP_PROP_CHANNEL_TARGET_HANDLE_TYPE,
             NULL) != TP_HANDLE_TYPE_CONTACT)
         {
           /* not interesting to this channel manager: we only care about
@@ -1305,15 +1312,15 @@ gabble_media_factory_represent_client (GabbleCapsChannelManager *manager,
         }
 
       if (tp_asv_get_boolean (filter,
-            TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialAudio", NULL)
+            TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_AUDIO, NULL)
           || tp_asv_get_boolean (filter,
-            TPY_IFACE_CHANNEL_TYPE_CALL ".InitialAudio", NULL))
+            TP_PROP_CHANNEL_TYPE_CALL_INITIAL_AUDIO, NULL))
         audio = TRUE;
 
       if (tp_asv_get_boolean (filter,
-            TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA ".InitialVideo", NULL)
+            TP_PROP_CHANNEL_TYPE_STREAMED_MEDIA_INITIAL_VIDEO, NULL)
           || tp_asv_get_boolean (filter,
-            TPY_IFACE_CHANNEL_TYPE_CALL ".InitialVideo", NULL))
+            TP_PROP_CHANNEL_TYPE_CALL_INITIAL_VIDEO, NULL))
         video = TRUE;
 
       /* If we've picked up all the capabilities we're ever going to, then
