@@ -73,7 +73,8 @@ static guint signals[LAST_SIGNAL] = {0};
 /* properties */
 enum
 {
-  PROP_OBJECT_PATH = 1,
+  PROP_DBUS_DAEMON = 1,
+  PROP_OBJECT_PATH,
   PROP_NAME,
   PROP_ID,
   PROP_MEDIA_TYPE,
@@ -97,6 +98,7 @@ struct _GabbleMediaStreamPrivate
   GabbleJingleContent *content;
 
   GabbleMediaSessionMode mode;
+  TpDBusDaemon *dbus_daemon;
   gchar *object_path;
   guint id;
   guint media_type;
@@ -159,7 +161,9 @@ static void update_direction (GabbleMediaStream *stream, GabbleJingleContent *c)
 static void update_sending (GabbleMediaStream *stream, gboolean start_sending);
 
 GabbleMediaStream *
-gabble_media_stream_new (const gchar *object_path,
+gabble_media_stream_new (
+    TpDBusDaemon *dbus_daemon,
+    const gchar *object_path,
     GabbleJingleContent *content,
     const gchar *name,
     guint id,
@@ -179,6 +183,7 @@ gabble_media_stream_new (const gchar *object_path,
     }
 
   result = g_object_new (GABBLE_TYPE_MEDIA_STREAM,
+      "dbus-daemon", dbus_daemon,
       "object-path", object_path,
       "content", content,
       "name", name,
@@ -267,8 +272,7 @@ gabble_media_stream_constructor (GType type, guint n_props,
   GObject *obj;
   GabbleMediaStream *stream;
   GabbleMediaStreamPrivate *priv;
-  TpDBusDaemon *bus;
-  GabbleConnection *connection;
+  GabbleJingleFactory *jf;
   gchar *stun_server;
   guint stun_port;
 
@@ -283,12 +287,11 @@ gabble_media_stream_constructor (GType type, guint n_props,
   /* STUN servers are needed as soon as the stream appears, so there's little
    * point in waiting for them - either they've already been resolved, or
    * we're too late to use them for this stream */
-  g_object_get (priv->content,
-      "connection", &connection,
-      NULL);
+  jf = gabble_jingle_session_get_factory (priv->content->session);
 
   /* maybe one day we'll support multiple STUN servers */
-  if (gabble_jingle_factory_get_stun_server (connection->jingle_factory,
+  if (gabble_jingle_info_get_stun_server (
+        gabble_jingle_factory_get_jingle_info (jf),
         &stun_server, &stun_port))
     {
       GValueArray *va = g_value_array_new (2);
@@ -303,10 +306,8 @@ gabble_media_stream_constructor (GType type, guint n_props,
     }
 
   /* go for the bus */
-  bus = tp_base_connection_get_dbus_daemon ((TpBaseConnection *) connection);
-  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
-
-  g_object_unref (connection);
+  g_assert (priv->dbus_daemon != NULL);
+  tp_dbus_daemon_register_object (priv->dbus_daemon, priv->object_path, obj);
 
   update_direction (stream, priv->content);
 
@@ -343,6 +344,9 @@ gabble_media_stream_get_property (GObject    *object,
   GabbleMediaStreamPrivate *priv = stream->priv;
 
   switch (property_id) {
+    case PROP_DBUS_DAEMON:
+      g_value_set_object (value, priv->dbus_daemon);
+      break;
     case PROP_OBJECT_PATH:
       g_value_set_string (value, priv->object_path);
       break;
@@ -405,6 +409,10 @@ gabble_media_stream_set_property (GObject      *object,
       g_free (priv->object_path);
       priv->object_path = g_value_dup_string (value);
       break;
+    case PROP_DBUS_DAEMON:
+      g_assert (priv->dbus_daemon == NULL);
+      priv->dbus_daemon = g_value_dup_object (value);
+      break;
     case PROP_NAME:
       g_free (stream->name);
       stream->name = g_value_dup_string (value);
@@ -529,6 +537,12 @@ gabble_media_stream_class_init (GabbleMediaStreamClass *gabble_media_stream_clas
   object_class->dispose = gabble_media_stream_dispose;
   object_class->finalize = gabble_media_stream_finalize;
 
+  param_spec = g_param_spec_object ("dbus-daemon", "TpDBusDaemon",
+      "Bus on which to export this object",
+      TP_TYPE_DBUS_DAEMON,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_DBUS_DAEMON, param_spec);
+
   param_spec = g_param_spec_string ("object-path", "D-Bus object path",
                                     "The D-Bus object path used for this "
                                     "object on the bus.",
@@ -694,6 +708,7 @@ gabble_media_stream_dispose (GObject *object)
 
   tp_clear_object (&priv->content);
   tp_clear_pointer (&self->name, g_free);
+  g_clear_object (&priv->dbus_daemon);
 
   if (G_OBJECT_CLASS (gabble_media_stream_parent_class)->dispose)
     G_OBJECT_CLASS (gabble_media_stream_parent_class)->dispose (object);
@@ -743,15 +758,15 @@ gabble_media_stream_codec_choice (TpSvcMediaStreamHandler *iface,
 
 gboolean
 gabble_media_stream_error (GabbleMediaStream *self,
-                           guint errno,
+                           guint errnum,
                            const gchar *message,
                            GError **error)
 {
   g_assert (GABBLE_IS_MEDIA_STREAM (self));
 
   DEBUG ( "Media.StreamHandler::Error called, error %u (%s) -- emitting signal",
-      errno, message);
-  g_signal_emit (self, signals[ERROR], 0, errno, message);
+      errnum, message);
+  g_signal_emit (self, signals[ERROR], 0, errnum, message);
 
   return TRUE;
 }
@@ -765,14 +780,14 @@ gabble_media_stream_error (GabbleMediaStream *self,
  */
 static void
 gabble_media_stream_error_async (TpSvcMediaStreamHandler *iface,
-                                 guint errno,
+                                 guint errnum,
                                  const gchar *message,
                                  DBusGMethodInvocation *context)
 {
   GabbleMediaStream *self = GABBLE_MEDIA_STREAM (iface);
   GError *error = NULL;
 
-  if (gabble_media_stream_error (self, errno, message, &error))
+  if (gabble_media_stream_error (self, errnum, message, &error))
     {
       tp_svc_media_stream_handler_return_from_error (context);
     }
@@ -1031,6 +1046,7 @@ pass_local_codecs (GabbleMediaStream *stream,
   JingleMediaDescription *md;
   const GPtrArray *hdrexts;
   GHashTable *fbs;
+  GError *wocky_error = NULL;
 
   DEBUG ("putting list of %d supported codecs from stream-engine into cache",
       codecs->len);
@@ -1178,8 +1194,14 @@ pass_local_codecs (GabbleMediaStream *stream,
 
   jingle_media_description_simplify (md);
 
-  return jingle_media_rtp_set_local_media_description (
-      GABBLE_JINGLE_MEDIA_RTP (priv->content), md, ready, error);
+  if (jingle_media_rtp_set_local_media_description (
+          GABBLE_JINGLE_MEDIA_RTP (priv->content), md, ready, &wocky_error))
+    return TRUE;
+
+  g_set_error_literal (error, TP_ERRORS, TP_ERROR_INVALID_ARGUMENT,
+      wocky_error->message);
+  g_clear_error (&wocky_error);
+  return FALSE;
 }
 
 /**
