@@ -131,4 +131,7 @@
  */
 #define NS_FACEBOOK_MESSAGES "http://www.facebook.com/xmpp/messages"
 
+#define NS_TP_FT_METADATA_SERVICE "http://telepathy.freedesktop.org/xmpp/file-transfer-service"
+#define NS_TP_FT_METADATA       "http://telepathy.freedesktop.org/xmpp/file-transfer-metadata"
+
 #endif /* __GABBLE_NAMESPACES__H__ */
