@@ -37,12 +37,7 @@
 #include <dbus/dbus-glib.h>
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/intset.h>
-#include <wocky/wocky-caps-cache.h>
-#include <wocky/wocky-caps-hash.h>
-#include <wocky/wocky-disco-identity.h>
-#include <wocky/wocky-utils.h>
-#include <wocky/wocky-namespaces.h>
-#include <wocky/wocky-data-form.h>
+#include <wocky/wocky.h>
 
 #define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
 
@@ -95,8 +90,8 @@ struct _GabblePresenceCachePrivate
   GabbleConnection *conn;
 
   gulong status_changed_cb;
-  LmMessageHandler *lm_message_cb;
-  LmMessageHandler *lm_presence_cb;
+  guint message_cb;
+  guint presence_cb;
 
   GHashTable *presence;
   TpHandleSet *presence_handles;
@@ -364,10 +359,20 @@ static void gabble_presence_cache_get_property (GObject *object, guint
 static GabblePresence *_cache_insert (GabblePresenceCache *cache,
     TpHandle handle);
 
+static void gabble_presence_cache_porter_available_cb (
+    GabbleConnection *conn,
+    WockyPorter *porter,
+    gpointer user_data);
 static void gabble_presence_cache_status_changed_cb (GabbleConnection *,
     TpConnectionStatus, TpConnectionStatusReason, gpointer);
-static LmHandlerResult gabble_presence_cache_lm_message_cb (LmMessageHandler*,
-    LmConnection*, LmMessage*, gpointer);
+static gboolean _parse_message_message (
+    WockyPorter *porter,
+    WockyStanza *message,
+    gpointer user_data);
+static gboolean gabble_presence_cache_presence_cb (
+    WockyPorter *porter,
+    WockyStanza *message,
+    gpointer user_data);
 
 static void
 gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
@@ -502,7 +507,9 @@ gabble_presence_cache_add_bundles (GabblePresenceCache *cache)
   gabble_presence_cache_add_bundle_caps (cache, \
       "http://talk.google.com/xmpp/client/caps#" cap, features); \
   gabble_presence_cache_add_bundle_caps (cache, \
-      "http://www.android.com/gtalk/client/caps#" cap, features);
+      "http://www.android.com/gtalk/client/caps#" cap, features); \
+  gabble_presence_cache_add_bundle_caps (cache, \
+      "http://talk.google.com/xmpp/bot/caps#" cap, features);
 
   /* Cache various bundle from the Google Talk clients as trusted.  Some old
    * versions of Google Talk do not reply correctly to discovery requests.
@@ -562,6 +569,8 @@ gabble_presence_cache_constructor (GType type, guint n_props,
 
   priv->status_changed_cb = g_signal_connect (priv->conn, "status-changed",
       G_CALLBACK (gabble_presence_cache_status_changed_cb), obj);
+  tp_g_signal_connect_object (priv->conn, "porter-available",
+      G_CALLBACK (gabble_presence_cache_porter_available_cb), obj, 0);
 
   return obj;
 }
@@ -588,8 +597,8 @@ gabble_presence_cache_dispose (GObject *object)
   tp_clear_pointer (&priv->decloak_requests, g_hash_table_unref);
   tp_clear_pointer (&priv->decloak_handles, tp_handle_set_destroy);
 
-  g_assert (priv->lm_message_cb == NULL);
-  g_assert (priv->lm_presence_cb == NULL);
+  g_assert (priv->message_cb == 0);
+  g_assert (priv->presence_cb == 0);
 
   g_signal_handler_disconnect (priv->conn, priv->status_changed_cb);
 
@@ -659,6 +668,27 @@ gabble_presence_cache_set_property (GObject     *object,
   }
 }
 
+static void
+gabble_presence_cache_porter_available_cb (
+    GabbleConnection *conn,
+    WockyPorter *porter,
+    gpointer user_data)
+{
+  GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (user_data);
+  GabblePresenceCachePrivate *priv = cache->priv;
+
+  priv->message_cb = wocky_porter_register_handler_from_anyone (porter,
+      WOCKY_STANZA_TYPE_MESSAGE, WOCKY_STANZA_SUB_TYPE_NONE,
+      WOCKY_PORTER_HANDLER_PRIORITY_MAX,
+      _parse_message_message, cache,
+      NULL);
+  priv->presence_cb = wocky_porter_register_handler_from_anyone (porter,
+      WOCKY_STANZA_TYPE_PRESENCE, WOCKY_STANZA_SUB_TYPE_NONE,
+      WOCKY_PORTER_HANDLER_PRIORITY_MIN,
+      gabble_presence_cache_presence_cb, cache,
+      NULL);
+}
+
 static void
 gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
                                          TpConnectionStatus status,
@@ -673,25 +703,6 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
   switch (status)
     {
     case TP_CONNECTION_STATUS_CONNECTING:
-      g_assert (priv->lm_message_cb == NULL);
-      g_assert (priv->lm_presence_cb == NULL);
-
-      /* these are separate despite having the same callback and user_data,
-       * because the Wocky fake-Loudmouth compat layer only lets you register
-       * each handler once */
-      priv->lm_message_cb = lm_message_handler_new (
-          gabble_presence_cache_lm_message_cb, cache, NULL);
-      priv->lm_presence_cb = lm_message_handler_new (
-          gabble_presence_cache_lm_message_cb, cache, NULL);
-
-      lm_connection_register_message_handler (priv->conn->lmconn,
-                                              priv->lm_presence_cb,
-                                              LM_MESSAGE_TYPE_PRESENCE,
-                                              LM_HANDLER_PRIORITY_LAST);
-      lm_connection_register_message_handler (priv->conn->lmconn,
-                                              priv->lm_message_cb,
-                                              LM_MESSAGE_TYPE_MESSAGE,
-                                              LM_HANDLER_PRIORITY_FIRST);
       break;
 
     case TP_CONNECTION_STATUS_CONNECTED:
@@ -702,16 +713,20 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
       break;
 
     case TP_CONNECTION_STATUS_DISCONNECTED:
-      if (priv->lm_message_cb != NULL)
-        lm_connection_unregister_message_handler (conn->lmconn,
-            priv->lm_message_cb, LM_MESSAGE_TYPE_MESSAGE);
+      if (conn->session != NULL)
+        {
+          WockyPorter *porter = wocky_session_get_porter (conn->session);
 
-      if (priv->lm_presence_cb != NULL)
-        lm_connection_unregister_message_handler (conn->lmconn,
-            priv->lm_presence_cb, LM_MESSAGE_TYPE_PRESENCE);
+          if (priv->message_cb != 0)
+            wocky_porter_unregister_handler (porter, priv->message_cb);
+
+          if (priv->presence_cb != 0)
+            wocky_porter_unregister_handler (porter, priv->presence_cb);
+
+          priv->message_cb = 0;
+          priv->presence_cb = 0;
+        }
 
-      tp_clear_pointer (&priv->lm_message_cb, lm_message_handler_unref);
-      tp_clear_pointer (&priv->lm_presence_cb, lm_message_handler_unref);
       break;
 
     default:
@@ -720,22 +735,10 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
 }
 
 static GabblePresenceId
-_presence_node_get_status (LmMessageNode *pres_node)
+_presence_node_get_status (WockyNode *pres_node)
 {
-  const gchar *presence_show;
-  LmMessageNode *child_node = lm_message_node_get_child (pres_node, "show");
-
-  if (!child_node)
-    {
-      /*
-      NODE_DEBUG (pres_node,
-        "<presence> without <show> received from server, "
-        "setting presence to available");
-      */
-      return GABBLE_PRESENCE_AVAILABLE;
-    }
-
-  presence_show = lm_message_node_get_value (child_node);
+  const gchar *presence_show =
+      wocky_node_get_content_from_child (pres_node, "show");
 
   if (!presence_show)
     {
@@ -768,12 +771,12 @@ static void
 _grab_nickname (GabblePresenceCache *cache,
                 TpHandle handle,
                 const gchar *from,
-                LmMessageNode *node)
+                WockyNode *node)
 {
   const gchar *nickname;
   GabblePresence *presence;
 
-  node = lm_message_node_get_child_with_namespace (node, "nick", NS_NICK);
+  node = wocky_node_get_child_ns (node, "nick", NS_NICK);
 
   if (NULL == node)
     return;
@@ -783,7 +786,7 @@ _grab_nickname (GabblePresenceCache *cache,
   if (NULL == presence)
     return;
 
-  nickname = lm_message_node_get_value (node);
+  nickname = node->content;
   DEBUG ("got nickname \"%s\" for %s", nickname, from);
 
   if (tp_strdiff (presence->nickname, nickname))
@@ -798,7 +801,7 @@ static void
 self_vcard_request_cb (GabbleVCardManager *self,
                        GabbleVCardManagerRequest *request,
                        TpHandle handle,
-                       LmMessageNode *vcard,
+                       WockyNode *vcard,
                        GError *error,
                        gpointer user_data)
 {
@@ -887,12 +890,12 @@ static void
 _grab_avatar_sha1 (GabblePresenceCache *cache,
                    TpHandle handle,
                    const gchar *from,
-                   LmMessageNode *node)
+                   WockyNode *node)
 {
   GabblePresenceCachePrivate *priv = cache->priv;
   TpBaseConnection *base_conn = (TpBaseConnection *) priv->conn;
   const gchar *sha1;
-  LmMessageNode *x_node, *photo_node;
+  WockyNode *x_node, *photo_node;
   GabblePresence *presence;
 
   if (handle == base_conn->self_handle)
@@ -903,7 +906,7 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
   if (NULL == presence)
     return;
 
-  x_node = lm_message_node_get_child_with_namespace (node, "x",
+  x_node = wocky_node_get_child_ns (node, "x",
       NS_VCARD_TEMP_UPDATE);
 
   if (NULL == x_node)
@@ -925,14 +928,14 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
       return;
     }
 
-  photo_node = lm_message_node_get_child (x_node, "photo");
+  photo_node = wocky_node_get_child (x_node, "photo");
 
   /* If there is no photo node, the resource supports XEP-0153, but has
    * nothing in particular to say about the avatar. */
   if (NULL == photo_node)
     return;
 
-  sha1 = lm_message_node_get_value (photo_node);
+  sha1 = photo_node->content;
 
   /* "" means we know there is no avatar. NULL means we don't know what is the
    * avatar. In this case, there is a <photo> node. */
@@ -960,30 +963,30 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
 
 static GSList *
 _parse_cap_bundles (
-    LmMessageNode *lm_node,
+    WockyNode *lm_node,
     const gchar **hash,
     const gchar **ver)
 {
   const gchar *node, *ext;
   GSList *uris = NULL;
-  LmMessageNode *cap_node;
+  WockyNode *cap_node;
 
   *hash = NULL;
   *ver = NULL;
 
-  cap_node = lm_message_node_get_child_with_namespace (lm_node, "c", NS_CAPS);
+  cap_node = wocky_node_get_child_ns (lm_node, "c", NS_CAPS);
 
   if (NULL == cap_node)
       return NULL;
 
-  *hash = lm_message_node_get_attribute (cap_node, "hash");
+  *hash = wocky_node_get_attribute (cap_node, "hash");
 
-  node = lm_message_node_get_attribute (cap_node, "node");
+  node = wocky_node_get_attribute (cap_node, "node");
 
   if (NULL == node)
     return NULL;
 
-  *ver = lm_message_node_get_attribute (cap_node, "ver");
+  *ver = wocky_node_get_attribute (cap_node, "ver");
 
   if (NULL != *ver)
     uris = g_slist_prepend (uris, g_strdup_printf ("%s#%s", node, *ver));
@@ -993,7 +996,7 @@ _parse_cap_bundles (
   if (NULL != *hash)
     return uris;
 
-  ext = lm_message_node_get_attribute (cap_node, "ext");
+  ext = wocky_node_get_attribute (cap_node, "ext");
 
   if (NULL != ext)
     {
@@ -1012,19 +1015,19 @@ _parse_cap_bundles (
 
 static void
 _parse_node (GabblePresence *presence,
-    LmMessageNode *lm_node,
+    WockyNode *lm_node,
     const gchar *resource,
     guint serial)
 {
-  LmMessageNode *cap_node;
+  WockyNode *cap_node;
   const gchar *node;
 
-  cap_node = lm_message_node_get_child_with_namespace (lm_node, "c", NS_CAPS);
+  cap_node = wocky_node_get_child_ns (lm_node, "c", NS_CAPS);
 
   if (NULL == cap_node)
     return;
 
-  node = lm_message_node_get_attribute (cap_node, "node");
+  node = wocky_node_get_attribute (cap_node, "node");
 
   if (!tp_strdiff (node, "http://mail.google.com/xmpp/client/caps"))
     {
@@ -1044,7 +1047,7 @@ static void _caps_disco_cb (GabbleDisco *disco,
     GabbleDiscoRequest *request,
     const gchar *jid,
     const gchar *node,
-    LmMessageNode *query_result,
+    WockyNode *query_result,
     GError *error,
     gpointer user_data);
 
@@ -1200,7 +1203,7 @@ emit_capabilities_discovered (GabblePresenceCache *cache,
 
 static guint
 client_types_from_message (TpHandle handle,
-    LmMessageNode *lm_node,
+    WockyNode *lm_node,
     const gchar *resource)
 {
   WockyNode *identity, *query_result = (WockyNode *) lm_node;
@@ -1285,7 +1288,7 @@ _caps_disco_cb (GabbleDisco *disco,
                 GabbleDiscoRequest *request,
                 const gchar *jid,
                 const gchar *node,
-                LmMessageNode *query_result,
+                WockyNode *query_result,
                 GError *error,
                 gpointer user_data)
 {
@@ -1648,7 +1651,7 @@ _process_caps (GabblePresenceCache *cache,
                GabblePresence *presence,
                TpHandle handle,
                const gchar *from,
-               LmMessageNode *lm_node)
+               WockyNode *lm_node)
 {
   const gchar *resource;
   GSList *uris, *i;
@@ -1710,18 +1713,71 @@ _process_caps (GabblePresenceCache *cache,
   g_slist_free (uris);
 }
 
-LmHandlerResult
-gabble_presence_parse_presence_message (GabblePresenceCache *cache,
-                         TpHandle handle,
-                         const gchar *from,
-                         LmMessage *message)
+static void
+presence_cache_check_for_decloak_request (
+    GabblePresenceCache *cache,
+    WockyStanza *stanza,
+    TpHandle handle,
+    const gchar *from)
 {
   GabblePresenceCachePrivate *priv = cache->priv;
+  WockyNode *presence_node = wocky_stanza_get_top_node (stanza);
+  WockyNode *child_node;
+
+  /* If we receive (directed or broadcast) presence of any sort from someone,
+   * it counts as a reply to any pending de-cloak request we might have been
+   * tracking */
+  g_hash_table_remove (priv->decloak_requests, GUINT_TO_POINTER (handle));
+
+  child_node = wocky_node_get_child_ns (presence_node, "temppres",
+      NS_TEMPPRES);
+
+  if (child_node != NULL)
+    {
+      gboolean decloak;
+      const gchar *reason;
+
+      /* this is a request to de-cloak, i.e. leak a minimal version of our
+       * presence to the peer */
+      g_object_get (priv->conn,
+          "decloak-automatically", &decloak,
+          NULL);
+
+      reason = wocky_node_get_attribute (child_node, "reason");
+
+      if (reason == NULL)
+        reason = "";
+
+      DEBUG ("Considering whether to decloak, reason='%s', conclusion=%d",
+          reason, decloak);
+
+      conn_decloak_emit_requested (priv->conn, handle, reason, decloak);
+
+      if (decloak)
+        gabble_connection_send_capabilities (priv->conn, from, NULL);
+    }
+
+}
+
+
+/* FIXME: in a cruel twist of fate, this is called by GabbleMucChannel!
+ * Presumably this is because the handler priority here is MIN, so WockyMuc
+ * steals the presence stanza before we can scrape our information out of it?
+ */
+gboolean
+gabble_presence_parse_presence_message (
+    GabblePresenceCache *cache,
+    TpHandle handle,
+    const gchar *from,
+    WockyStanza *message)
+{
+  GabblePresenceCachePrivate *priv = cache->priv;
+  const gchar *prio;
   gint8 priority = 0;
   const gchar *resource, *status_message = NULL;
   gchar *my_full_jid;
-  LmMessageNode *presence_node, *child_node;
-  LmHandlerResult ret = LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+  WockyNode *presence_node;
+  WockyStanzaSubType sub_type;
   GabblePresenceId presence_id;
   GabblePresence *presence;
 
@@ -1733,7 +1789,7 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
   if (!tp_strdiff (from, my_full_jid))
     {
       g_free (my_full_jid);
-      return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+      return TRUE;
     }
   g_free (my_full_jid);
 
@@ -1751,61 +1807,19 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
        * presence around when it's unavailable. */
       presence->keep_unavailable = FALSE;
 
-  /* If we receive (directed or broadcast) presence of any sort from someone,
-   * it counts as a reply to any pending de-cloak request we might have been
-   * tracking */
-  g_hash_table_remove (priv->decloak_requests, GUINT_TO_POINTER (handle));
+  status_message = wocky_node_get_content_from_child (presence_node, "status");
+  prio = wocky_node_get_content_from_child (presence_node, "priority");
 
-  child_node = lm_message_node_get_child (presence_node, "status");
+  if (prio != NULL)
+    priority = CLAMP (atoi (prio), G_MININT8, G_MAXINT8);
 
-  if (child_node)
-    status_message = lm_message_node_get_value (child_node);
+  presence_cache_check_for_decloak_request (cache, message, handle, from);
 
-  if (child_node)
-    status_message = lm_message_node_get_value (child_node);
-
-  child_node = lm_message_node_get_child (presence_node, "priority");
-
-  if (child_node)
+  wocky_stanza_get_type_info (message, NULL, &sub_type);
+  switch (sub_type)
     {
-      const gchar *prio = lm_message_node_get_value (child_node);
-
-      if (prio != NULL)
-        priority = CLAMP (atoi (prio), G_MININT8, G_MAXINT8);
-    }
-
-  child_node = wocky_node_get_child_ns (presence_node, "temppres",
-      NS_TEMPPRES);
-
-  if (child_node != NULL)
-    {
-      gboolean decloak;
-      const gchar *reason;
-
-      /* this is a request to de-cloak, i.e. leak a minimal version of our
-       * presence to the peer */
-      g_object_get (priv->conn,
-          "decloak-automatically", &decloak,
-          NULL);
-
-      reason = lm_message_node_get_attribute (child_node, "reason");
-
-      if (reason == NULL)
-        reason = "";
-
-      DEBUG ("Considering whether to decloak, reason='%s', conclusion=%d",
-          reason, decloak);
-
-      conn_decloak_emit_requested (priv->conn, handle, reason, decloak);
-
-      if (decloak)
-        gabble_connection_send_capabilities (priv->conn, from, NULL);
-    }
-
-  switch (lm_message_get_sub_type (message))
-    {
-    case LM_MESSAGE_SUB_TYPE_NOT_SET:
-    case LM_MESSAGE_SUB_TYPE_AVAILABLE:
+    case WOCKY_STANZA_SUB_TYPE_NONE:
+    case WOCKY_STANZA_SUB_TYPE_AVAILABLE:
       presence_id = _presence_node_get_status (presence_node);
       gabble_presence_cache_update (cache, handle, resource, presence_id,
           status_message, priority);
@@ -1817,18 +1831,34 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
       _grab_avatar_sha1 (cache, handle, from, presence_node);
       _process_caps (cache, presence, handle, from, presence_node);
 
-      ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
-      break;
+      return TRUE;
+
+    case WOCKY_STANZA_SUB_TYPE_ERROR:
+    {
+      GError *error = NULL;
+      gboolean ret;
 
-    case LM_MESSAGE_SUB_TYPE_ERROR:
       NODE_DEBUG (presence_node, "Received error presence");
+
+      ret = wocky_stanza_extract_errors (message, NULL, &error, NULL, NULL);
+      g_assert (ret);
+
+      /* If there's a <status/> in this presence, it's our own echoed back at
+       * us. So we don't want to use that. Instead, we use the <error><text> if
+       * there is any, or the name of the error condition if not. */
+      if (tp_str_empty (error->message))
+        status_message = wocky_enum_to_nick (WOCKY_TYPE_XMPP_ERROR,
+            error->code);
+      else
+        status_message = error->message;
+
       gabble_presence_cache_update (cache, handle, resource,
           GABBLE_PRESENCE_ERROR, status_message, priority);
 
-      ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
-      break;
+      return TRUE;
+    }
 
-    case LM_MESSAGE_SUB_TYPE_UNAVAILABLE:
+    case WOCKY_STANZA_SUB_TYPE_UNAVAILABLE:
       if (gabble_roster_handle_sends_presence_to_us (priv->conn->roster,
             handle))
         presence_id = GABBLE_PRESENCE_OFFLINE;
@@ -1838,14 +1868,11 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
       gabble_presence_cache_update (cache, handle, resource,
           presence_id, status_message, priority);
 
-      ret = LM_HANDLER_RESULT_REMOVE_MESSAGE;
-      break;
+      return TRUE;
 
     default:
-      break;
+      return FALSE;
     }
-
-  return ret;
 }
 
 /* FIXME: this scrapes nicknames out of <messages>, and relies on im-channel.c
@@ -1858,24 +1885,45 @@ gabble_presence_parse_presence_message (GabblePresenceCache *cache,
  * to go away when the channel closes, rather than relying on this
  * spooky-action-at-a-distance.
  */
-static LmHandlerResult
-_parse_message_message (GabblePresenceCache *cache,
-                        TpHandle handle,
-                        const gchar *from,
-                        LmMessage *message)
+static gboolean
+_parse_message_message (
+    WockyPorter *porter,
+    WockyStanza *message,
+    gpointer user_data)
 {
-  LmMessageNode *node;
+  GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (user_data);
+  GabblePresenceCachePrivate *priv = cache->priv;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  const gchar *from = wocky_stanza_get_from (message);
+  TpHandle handle;
+  WockyStanzaSubType sub_type;
+  WockyNode *node;
   GabblePresence *presence;
 
-  switch (lm_message_get_sub_type (message))
+  if (NULL == from)
     {
-    case LM_MESSAGE_SUB_TYPE_NOT_SET:
-    case LM_MESSAGE_SUB_TYPE_NORMAL:
-    case LM_MESSAGE_SUB_TYPE_CHAT:
-    case LM_MESSAGE_SUB_TYPE_GROUPCHAT:
+      STANZA_DEBUG (message, "message without from attribute, ignoring");
+      return FALSE;
+    }
+
+  handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
+  if (0 == handle)
+    {
+      STANZA_DEBUG (message, "ignoring message from malformed jid");
+      return FALSE;
+    }
+
+  wocky_stanza_get_type_info (message, NULL, &sub_type);
+  switch (sub_type)
+    {
+    case WOCKY_STANZA_SUB_TYPE_NONE:
+    case WOCKY_STANZA_SUB_TYPE_NORMAL:
+    case WOCKY_STANZA_SUB_TYPE_CHAT:
+    case WOCKY_STANZA_SUB_TYPE_GROUPCHAT:
       break;
     default:
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      return FALSE;
     }
 
   presence = gabble_presence_cache_get (cache, handle);
@@ -1886,72 +1934,46 @@ _parse_message_message (GabblePresenceCache *cache,
       presence->keep_unavailable = TRUE;
     }
 
-  node = lm_message_get_node (message);
+  node = wocky_stanza_get_top_node (message);
 
   _grab_nickname (cache, handle, from, node);
 
-  return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+  return FALSE;
 }
 
 
-/**
- * gabble_presence_cache_lm_message_cb:
- * @handler: #LmMessageHandler for this message
- * @connection: #LmConnection that originated the message
- * @message: the presence message
- * @user_data: callback data
+/*
+ * gabble_presence_cache_presence_cb:
  *
- * Called by loudmouth when we get an incoming <presence>.
+ * Called by Wocky when we get an incoming <presence>.
  */
-static LmHandlerResult
-gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
-                                     LmConnection *lmconn,
-                                     LmMessage *message,
-                                     gpointer user_data)
+static gboolean
+gabble_presence_cache_presence_cb (
+    WockyPorter *porter,
+    WockyStanza *message,
+    gpointer user_data)
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (user_data);
   GabblePresenceCachePrivate *priv = cache->priv;
   TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
       (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
-  const char *from;
-  LmHandlerResult ret;
+  const char *from = wocky_stanza_get_from (message);
   TpHandle handle;
 
-  g_assert (lmconn == priv->conn->lmconn);
-
-  from = lm_message_node_get_attribute (wocky_stanza_get_top_node (message),
-      "from");
-
   if (NULL == from)
     {
       STANZA_DEBUG (message, "message without from attribute, ignoring");
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      return FALSE;
     }
 
   handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
-
   if (0 == handle)
     {
       STANZA_DEBUG (message, "ignoring message from malformed jid");
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      return FALSE;
     }
 
-  switch (lm_message_get_type (message))
-    {
-    case LM_MESSAGE_TYPE_PRESENCE:
-      ret = gabble_presence_parse_presence_message (cache, handle,
-        from, message);
-      break;
-    case LM_MESSAGE_TYPE_MESSAGE:
-      ret = _parse_message_message (cache, handle, from, message);
-      break;
-    default:
-      ret = LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
-      break;
-    }
-
-  tp_handle_unref (contact_repo, handle);
-  return ret;
+  return gabble_presence_parse_presence_message (cache, handle, from, message);
 }
 
 
@@ -1975,6 +1997,17 @@ gabble_presence_cache_get (GabblePresenceCache *cache, TpHandle handle)
   return g_hash_table_lookup (priv->presence, GUINT_TO_POINTER (handle));
 }
 
+GabblePresence *
+gabble_presence_cache_get_for_contact (
+    GabblePresenceCache *cache,
+    WockyContact *contact)
+{
+  GabblePresenceCachePrivate *priv = cache->priv;
+  TpHandle handle = ensure_handle_from_contact (priv->conn, contact);
+
+  return g_hash_table_lookup (priv->presence, GUINT_TO_POINTER (handle));
+}
+
 void
 gabble_presence_cache_maybe_remove (
     GabblePresenceCache *cache,
