@@ -99,6 +99,9 @@ GType gabble_presence_cache_get_type (void);
 GabblePresenceCache *gabble_presence_cache_new (GabbleConnection *conn);
 GabblePresence *gabble_presence_cache_get (GabblePresenceCache *cache,
     TpHandle handle);
+GabblePresence *gabble_presence_cache_get_for_contact (
+    GabblePresenceCache *cache,
+    WockyContact *contact);
 void gabble_presence_cache_update (GabblePresenceCache *cache,
     TpHandle handle, const gchar *resource, GabblePresenceId presence_id,
     const gchar *status_message, gint8 priority);
@@ -118,11 +121,11 @@ const GabbleCapabilityInfo *gabble_presence_cache_peek_own_caps (
 void gabble_presence_cache_really_remove (GabblePresenceCache *cache,
     TpHandle handle);
 
-LmHandlerResult
-gabble_presence_parse_presence_message (GabblePresenceCache *cache,
+gboolean gabble_presence_parse_presence_message (
+    GabblePresenceCache *cache,
     TpHandle handle,
     const gchar *from,
-    LmMessage *message);
+    WockyStanza *message);
 
 void gabble_presence_cache_contacts_added_to_olpc_view (
     GabblePresenceCache *cache, TpHandleSet *handles);
