@@ -24,6 +24,7 @@
 
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/base-channel.h>
 
 #include "connection.h"
 #include "extensions/extensions.h"
@@ -36,13 +37,13 @@ typedef struct _GabbleTubeStreamPrivate GabbleTubeStreamPrivate;
 typedef struct _GabbleTubeStreamClass GabbleTubeStreamClass;
 
 struct _GabbleTubeStreamClass {
-  GObjectClass parent_class;
+  TpBaseChannelClass parent_class;
 
   TpDBusPropertiesMixinClass dbus_props_class;
 };
 
 struct _GabbleTubeStream {
-  GObject parent;
+  TpBaseChannel parent;
 
   GabbleTubeStreamPrivate *priv;
 };
@@ -68,7 +69,7 @@ GType gabble_tube_stream_get_type (void);
 GabbleTubeStream *gabble_tube_stream_new (GabbleConnection *conn,
     TpHandle handle, TpHandleType handle_type, TpHandle self_handle,
     TpHandle initiator, const gchar *service, GHashTable *parameters,
-    guint id, GabbleMucChannel *muc);
+    guint id, GabbleMucChannel *muc, gboolean requested);
 
 gboolean gabble_tube_stream_check_params (TpSocketAddressType address_type,
     const GValue *address, TpSocketAccessControl access_control,
