@@ -10,6 +10,12 @@ from servicetest import EventPattern, assertEquals, assertNotEquals
 import ns
 import constants as cs
 
+from config import VOIP_ENABLED
+
+if not VOIP_ENABLED:
+    print "NOTE: built with --disable-voip"
+    raise SystemExit(77)
+
 def test(q, bus, conn, stream):
     self_presence = q.expect('stream-presence')
 
